@@ -1,3 +1,6 @@
+#ifndef _LATER_TIMECONV_H_
+#define _LATER_TIMECONV_H_
+
 #include <sys/time.h>
 // Some platforms (Win32, previously some Mac versions) use
 // tinycthread.h to provide timespec. Whether tinycthread
@@ -33,3 +36,5 @@ inline timespec addSeconds(const timespec& time, double secs) {
   }
   return ts;
 }
+
+#endif // _LATER_TIMECONV_H_
